# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.banner import Banner
from kestrapy.models.edition_provider_edition import EditionProviderEdition
from kestrapy.models.left_sidebar_configuration import LeftSidebarConfiguration
from kestrapy.models.misc_controller_environment import MiscControllerEnvironment
from kestrapy.models.misc_controller_plugin_id_and_version import MiscControllerPluginIdAndVersion
from kestrapy.models.misc_controller_preview import MiscControllerPreview
from kestrapy.models.misc_controller_tenant_configuration_info import MiscControllerTenantConfigurationInfo
from kestrapy.models.right_sidebar_configuration import RightSidebarConfiguration
from typing import Optional, Set
from typing_extensions import Self

class MiscControllerEEConfiguration(BaseModel):
    """
    MiscControllerEEConfiguration
    """ # noqa: E501
    uuid: Optional[StrictStr] = None
    version: Optional[StrictStr] = None
    edition: Optional[EditionProviderEdition] = None
    commit_id: Optional[StrictStr] = Field(default=None, alias="commitId")
    chart_default_duration: Optional[StrictStr] = Field(default=None, alias="chartDefaultDuration")
    commit_date: Optional[datetime] = Field(default=None, alias="commitDate")
    is_custom_dashboards_enabled: Optional[StrictBool] = Field(default=None, alias="isCustomDashboardsEnabled")
    is_anonymous_usage_enabled: Optional[StrictBool] = Field(default=None, alias="isAnonymousUsageEnabled")
    is_ui_anonymous_usage_enabled: Optional[StrictBool] = Field(default=None, alias="isUiAnonymousUsageEnabled")
    is_template_enabled: Optional[StrictBool] = Field(default=None, alias="isTemplateEnabled")
    environment: Optional[MiscControllerEnvironment] = None
    url: Optional[StrictStr] = None
    preview: Optional[MiscControllerPreview] = None
    system_namespace: Optional[StrictStr] = Field(default=None, alias="systemNamespace")
    hidden_labels_prefixes: Optional[List[StrictStr]] = Field(default=None, alias="hiddenLabelsPrefixes")
    is_ai_enabled: Optional[StrictBool] = Field(default=None, alias="isAiEnabled")
    is_basic_auth_initialized: Optional[StrictBool] = Field(default=None, alias="isBasicAuthInitialized")
    plugins_hash: Optional[StrictInt] = Field(default=None, alias="pluginsHash")
    is_concurrency_view_enabled: Optional[StrictBool] = Field(default=None, alias="isConcurrencyViewEnabled")
    tenants: Optional[MiscControllerTenantConfigurationInfo] = None
    secrets_enabled: Optional[StrictBool] = Field(default=None, alias="secretsEnabled")
    supported_storages: Optional[List[MiscControllerPluginIdAndVersion]] = Field(default=None, alias="supportedStorages")
    supported_secrets: Optional[List[MiscControllerPluginIdAndVersion]] = Field(default=None, alias="supportedSecrets")
    plugin_management_enabled: Optional[StrictBool] = Field(default=None, alias="pluginManagementEnabled")
    plugin_custom_enabled: Optional[StrictBool] = Field(default=None, alias="pluginCustomEnabled")
    banner: Optional[Banner] = None
    mail_service_enabled: Optional[StrictBool] = Field(default=None, alias="mailServiceEnabled")
    outputs_in_internal_storage_enabled: Optional[StrictBool] = Field(default=None, alias="outputsInInternalStorageEnabled")
    left_sidebar: Optional[LeftSidebarConfiguration] = Field(default=None, alias="leftSidebar")
    right_sidebar: Optional[RightSidebarConfiguration] = Field(default=None, alias="rightSidebar")
    in_maintenance: Optional[StrictBool] = Field(default=None, alias="inMaintenance")
    password_regexp: Optional[StrictStr] = Field(default=None, alias="passwordRegexp")
    passwordless_enabled: Optional[StrictBool] = Field(default=None, alias="passwordlessEnabled")
    airgapped: Optional[StrictBool] = None
    feature_gating: Optional[StrictBool] = Field(default=None, alias="featureGating")
    features: Optional[List[StrictStr]] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["uuid", "version", "edition", "commitId", "chartDefaultDuration", "commitDate", "isCustomDashboardsEnabled", "isAnonymousUsageEnabled", "isUiAnonymousUsageEnabled", "isTemplateEnabled", "environment", "url", "preview", "systemNamespace", "hiddenLabelsPrefixes", "isAiEnabled", "isBasicAuthInitialized", "pluginsHash", "isConcurrencyViewEnabled", "tenants", "secretsEnabled", "supportedStorages", "supportedSecrets", "pluginManagementEnabled", "pluginCustomEnabled", "banner", "mailServiceEnabled", "outputsInInternalStorageEnabled", "leftSidebar", "rightSidebar", "inMaintenance", "passwordRegexp", "passwordlessEnabled", "airgapped", "featureGating", "features"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of MiscControllerEEConfiguration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of environment
        if self.environment:
            _dict['environment'] = self.environment.to_dict()
        # override the default output from pydantic by calling `to_dict()` of preview
        if self.preview:
            _dict['preview'] = self.preview.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tenants
        if self.tenants:
            _dict['tenants'] = self.tenants.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in supported_storages (list)
        _items = []
        if self.supported_storages:
            for _item_supported_storages in self.supported_storages:
                if _item_supported_storages:
                    _items.append(_item_supported_storages.to_dict())
            _dict['supportedStorages'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in supported_secrets (list)
        _items = []
        if self.supported_secrets:
            for _item_supported_secrets in self.supported_secrets:
                if _item_supported_secrets:
                    _items.append(_item_supported_secrets.to_dict())
            _dict['supportedSecrets'] = _items
        # override the default output from pydantic by calling `to_dict()` of banner
        if self.banner:
            _dict['banner'] = self.banner.to_dict()
        # override the default output from pydantic by calling `to_dict()` of left_sidebar
        if self.left_sidebar:
            _dict['leftSidebar'] = self.left_sidebar.to_dict()
        # override the default output from pydantic by calling `to_dict()` of right_sidebar
        if self.right_sidebar:
            _dict['rightSidebar'] = self.right_sidebar.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of MiscControllerEEConfiguration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "uuid": obj.get("uuid"),
            "version": obj.get("version"),
            "edition": obj.get("edition"),
            "commitId": obj.get("commitId"),
            "chartDefaultDuration": obj.get("chartDefaultDuration"),
            "commitDate": obj.get("commitDate"),
            "isCustomDashboardsEnabled": obj.get("isCustomDashboardsEnabled"),
            "isAnonymousUsageEnabled": obj.get("isAnonymousUsageEnabled"),
            "isUiAnonymousUsageEnabled": obj.get("isUiAnonymousUsageEnabled"),
            "isTemplateEnabled": obj.get("isTemplateEnabled"),
            "environment": MiscControllerEnvironment.from_dict(obj["environment"]) if obj.get("environment") is not None else None,
            "url": obj.get("url"),
            "preview": MiscControllerPreview.from_dict(obj["preview"]) if obj.get("preview") is not None else None,
            "systemNamespace": obj.get("systemNamespace"),
            "hiddenLabelsPrefixes": obj.get("hiddenLabelsPrefixes"),
            "isAiEnabled": obj.get("isAiEnabled"),
            "isBasicAuthInitialized": obj.get("isBasicAuthInitialized"),
            "pluginsHash": obj.get("pluginsHash"),
            "isConcurrencyViewEnabled": obj.get("isConcurrencyViewEnabled"),
            "tenants": MiscControllerTenantConfigurationInfo.from_dict(obj["tenants"]) if obj.get("tenants") is not None else None,
            "secretsEnabled": obj.get("secretsEnabled"),
            "supportedStorages": [MiscControllerPluginIdAndVersion.from_dict(_item) for _item in obj["supportedStorages"]] if obj.get("supportedStorages") is not None else None,
            "supportedSecrets": [MiscControllerPluginIdAndVersion.from_dict(_item) for _item in obj["supportedSecrets"]] if obj.get("supportedSecrets") is not None else None,
            "pluginManagementEnabled": obj.get("pluginManagementEnabled"),
            "pluginCustomEnabled": obj.get("pluginCustomEnabled"),
            "banner": Banner.from_dict(obj["banner"]) if obj.get("banner") is not None else None,
            "mailServiceEnabled": obj.get("mailServiceEnabled"),
            "outputsInInternalStorageEnabled": obj.get("outputsInInternalStorageEnabled"),
            "leftSidebar": LeftSidebarConfiguration.from_dict(obj["leftSidebar"]) if obj.get("leftSidebar") is not None else None,
            "rightSidebar": RightSidebarConfiguration.from_dict(obj["rightSidebar"]) if obj.get("rightSidebar") is not None else None,
            "inMaintenance": obj.get("inMaintenance"),
            "passwordRegexp": obj.get("passwordRegexp"),
            "passwordlessEnabled": obj.get("passwordlessEnabled"),
            "airgapped": obj.get("airgapped"),
            "featureGating": obj.get("featureGating"),
            "features": obj.get("features")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from kestrapy.models.isolation import Isolation
from kestrapy.models.namespace_allowed_namespace import NamespaceAllowedNamespace
from kestrapy.models.plugin_default import PluginDefault
from kestrapy.models.worker_group import WorkerGroup
from typing import Optional, Set
from typing_extensions import Self

class Namespace(BaseModel):
    """
    Namespace
    """ # noqa: E501
    id: Annotated[str, Field(strict=True)]
    deleted: StrictBool
    storage_isolation: Optional[Isolation] = Field(default=None, alias="storageIsolation")
    secret_isolation: Optional[Isolation] = Field(default=None, alias="secretIsolation")
    description: Optional[StrictStr] = None
    variables: Optional[Dict[str, Dict[str, Any]]] = None
    plugin_defaults: Optional[List[PluginDefault]] = Field(default=None, alias="pluginDefaults")
    allowed_namespaces: Optional[List[NamespaceAllowedNamespace]] = Field(default=None, alias="allowedNamespaces")
    worker_group: Optional[WorkerGroup] = Field(default=None, alias="workerGroup")
    storage_type: Optional[StrictStr] = Field(default=None, alias="storageType")
    storage_configuration: Optional[Dict[str, Dict[str, Any]]] = Field(default=None, alias="storageConfiguration")
    secret_type: Optional[StrictStr] = Field(default=None, alias="secretType")
    secret_read_only: Optional[StrictBool] = Field(default=None, alias="secretReadOnly")
    secret_configuration: Optional[Dict[str, Dict[str, Any]]] = Field(default=None, alias="secretConfiguration")
    outputs_in_internal_storage: Optional[StrictBool] = Field(default=None, alias="outputsInInternalStorage")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["id", "deleted", "storageIsolation", "secretIsolation", "description", "variables", "pluginDefaults", "allowedNamespaces", "workerGroup", "storageType", "storageConfiguration", "secretType", "secretReadOnly", "secretConfiguration", "outputsInInternalStorage"]

    @field_validator('id')
    def id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9][a-z0-9._-]*", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9][a-z0-9._-]*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Namespace from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of storage_isolation
        if self.storage_isolation:
            _dict['storageIsolation'] = self.storage_isolation.to_dict()
        # override the default output from pydantic by calling `to_dict()` of secret_isolation
        if self.secret_isolation:
            _dict['secretIsolation'] = self.secret_isolation.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in plugin_defaults (list)
        _items = []
        if self.plugin_defaults:
            for _item_plugin_defaults in self.plugin_defaults:
                if _item_plugin_defaults:
                    _items.append(_item_plugin_defaults.to_dict())
            _dict['pluginDefaults'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in allowed_namespaces (list)
        _items = []
        if self.allowed_namespaces:
            for _item_allowed_namespaces in self.allowed_namespaces:
                if _item_allowed_namespaces:
                    _items.append(_item_allowed_namespaces.to_dict())
            _dict['allowedNamespaces'] = _items
        # override the default output from pydantic by calling `to_dict()` of worker_group
        if self.worker_group:
            _dict['workerGroup'] = self.worker_group.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Namespace from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "deleted": obj.get("deleted"),
            "storageIsolation": Isolation.from_dict(obj["storageIsolation"]) if obj.get("storageIsolation") is not None else None,
            "secretIsolation": Isolation.from_dict(obj["secretIsolation"]) if obj.get("secretIsolation") is not None else None,
            "description": obj.get("description"),
            "variables": obj.get("variables"),
            "pluginDefaults": [PluginDefault.from_dict(_item) for _item in obj["pluginDefaults"]] if obj.get("pluginDefaults") is not None else None,
            "allowedNamespaces": [NamespaceAllowedNamespace.from_dict(_item) for _item in obj["allowedNamespaces"]] if obj.get("allowedNamespaces") is not None else None,
            "workerGroup": WorkerGroup.from_dict(obj["workerGroup"]) if obj.get("workerGroup") is not None else None,
            "storageType": obj.get("storageType"),
            "storageConfiguration": obj.get("storageConfiguration"),
            "secretType": obj.get("secretType"),
            "secretReadOnly": obj.get("secretReadOnly"),
            "secretConfiguration": obj.get("secretConfiguration"),
            "outputsInInternalStorage": obj.get("outputsInInternalStorage")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class SchemaType(str, Enum):
    """
    SchemaType
    """

    """
    allowed enum values
    """
    FLOW = 'FLOW'
    TEMPLATE = 'TEMPLATE'
    TASK = 'TASK'
    TRIGGER = 'TRIGGER'
    PLUGINDEFAULT = 'PLUGINDEFAULT'
    APPS = 'APPS'
    TESTSUITES = 'TESTSUITES'
    DASHBOARD = 'DASHBOARD'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SchemaType from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from kestrapy.models.meta import Meta
from kestrapy.models.scim_extension import ScimExtension
from kestrapy.models.service_provider_configuration_authentication_schema import ServiceProviderConfigurationAuthenticationSchema
from kestrapy.models.service_provider_configuration_bulk_configuration import ServiceProviderConfigurationBulkConfiguration
from kestrapy.models.service_provider_configuration_filter_configuration import ServiceProviderConfigurationFilterConfiguration
from kestrapy.models.service_provider_configuration_supported_configuration import ServiceProviderConfigurationSupportedConfiguration
from typing import Optional, Set
from typing_extensions import Self

class ServiceProviderConfiguration(BaseModel):
    """
    ServiceProviderConfiguration
    """ # noqa: E501
    schemas: Optional[Annotated[List[StrictStr], Field(min_length=1)]] = None
    base_urn: Optional[StrictStr] = Field(default=None, alias="baseUrn")
    extensions: Optional[Dict[str, ScimExtension]] = None
    meta: Meta
    id: Optional[StrictStr] = None
    external_id: Optional[StrictStr] = Field(default=None, alias="externalId")
    resource_type: Optional[StrictStr] = Field(default=None, alias="resourceType")
    documentation_url: Optional[StrictStr] = Field(default=None, alias="documentationUrl")
    patch: Optional[ServiceProviderConfigurationSupportedConfiguration] = None
    bulk: Optional[ServiceProviderConfigurationBulkConfiguration] = None
    filter: Optional[ServiceProviderConfigurationFilterConfiguration] = None
    change_password: Optional[ServiceProviderConfigurationSupportedConfiguration] = Field(default=None, alias="changePassword")
    sort: Optional[ServiceProviderConfigurationSupportedConfiguration] = None
    etag: Optional[ServiceProviderConfigurationSupportedConfiguration] = None
    authentication_schemes: Optional[List[ServiceProviderConfigurationAuthenticationSchema]] = Field(default=None, alias="authenticationSchemes")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["schemas", "baseUrn", "extensions", "meta", "id", "externalId", "resourceType", "documentationUrl", "patch", "bulk", "filter", "changePassword", "sort", "etag", "authenticationSchemes"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ServiceProviderConfiguration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each value in extensions (dict)
        _field_dict = {}
        if self.extensions:
            for _key_extensions in self.extensions:
                if self.extensions[_key_extensions]:
                    _field_dict[_key_extensions] = self.extensions[_key_extensions].to_dict()
            _dict['extensions'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of meta
        if self.meta:
            _dict['meta'] = self.meta.to_dict()
        # override the default output from pydantic by calling `to_dict()` of patch
        if self.patch:
            _dict['patch'] = self.patch.to_dict()
        # override the default output from pydantic by calling `to_dict()` of bulk
        if self.bulk:
            _dict['bulk'] = self.bulk.to_dict()
        # override the default output from pydantic by calling `to_dict()` of filter
        if self.filter:
            _dict['filter'] = self.filter.to_dict()
        # override the default output from pydantic by calling `to_dict()` of change_password
        if self.change_password:
            _dict['changePassword'] = self.change_password.to_dict()
        # override the default output from pydantic by calling `to_dict()` of sort
        if self.sort:
            _dict['sort'] = self.sort.to_dict()
        # override the default output from pydantic by calling `to_dict()` of etag
        if self.etag:
            _dict['etag'] = self.etag.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in authentication_schemes (list)
        _items = []
        if self.authentication_schemes:
            for _item_authentication_schemes in self.authentication_schemes:
                if _item_authentication_schemes:
                    _items.append(_item_authentication_schemes.to_dict())
            _dict['authenticationSchemes'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ServiceProviderConfiguration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "schemas": obj.get("schemas"),
            "baseUrn": obj.get("baseUrn"),
            "extensions": dict(
                (_k, ScimExtension.from_dict(_v))
                for _k, _v in obj["extensions"].items()
            )
            if obj.get("extensions") is not None
            else None,
            "meta": Meta.from_dict(obj["meta"]) if obj.get("meta") is not None else None,
            "id": obj.get("id"),
            "externalId": obj.get("externalId"),
            "resourceType": obj.get("resourceType"),
            "documentationUrl": obj.get("documentationUrl"),
            "patch": ServiceProviderConfigurationSupportedConfiguration.from_dict(obj["patch"]) if obj.get("patch") is not None else None,
            "bulk": ServiceProviderConfigurationBulkConfiguration.from_dict(obj["bulk"]) if obj.get("bulk") is not None else None,
            "filter": ServiceProviderConfigurationFilterConfiguration.from_dict(obj["filter"]) if obj.get("filter") is not None else None,
            "changePassword": ServiceProviderConfigurationSupportedConfiguration.from_dict(obj["changePassword"]) if obj.get("changePassword") is not None else None,
            "sort": ServiceProviderConfigurationSupportedConfiguration.from_dict(obj["sort"]) if obj.get("sort") is not None else None,
            "etag": ServiceProviderConfigurationSupportedConfiguration.from_dict(obj["etag"]) if obj.get("etag") is not None else None,
            "authenticationSchemes": [ServiceProviderConfigurationAuthenticationSchema.from_dict(_item) for _item in obj["authenticationSchemes"]] if obj.get("authenticationSchemes") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


